using System;
using System.Text;
using System.Reflection;
using System.Collections;
using Server;
using Server.Targeting;

namespace Server.Scripts.Commands
{
	public class Add
	{
		public static void Initialize()
		{
			Server.Commands.Register( "Add", AccessLevel.GameMaster, new CommandEventHandler( Add_OnCommand ) );
			Server.Commands.Register( "Tile", AccessLevel.GameMaster, new CommandEventHandler( Tile_OnCommand ) );
			Server.Commands.Register( "TileRXYZ", AccessLevel.GameMaster, new CommandEventHandler( TileRXYZ_OnCommand ) );
			Server.Commands.Register( "TileXYZ", AccessLevel.GameMaster, new CommandEventHandler( TileXYZ_OnCommand ) );
			Server.Commands.Register( "TileZ", AccessLevel.GameMaster, new CommandEventHandler( TileZ_OnCommand ) );
		}

		public static void Invoke( Mobile from, Point3D start, Point3D end, string[] args )
		{
			StringBuilder sb = new StringBuilder();

			sb.AppendFormat( "{0} {1} building ", from.AccessLevel, CommandLogging.Format( from ) );

			if ( start == end )
				sb.AppendFormat( "at {0}", start );
			else
				sb.AppendFormat( "from {0} to {1}", start, end );

			sb.Append( ":" );

			for ( int i = 0; i < args.Length; ++i )
				sb.AppendFormat( " \"{0}\"", args[i] );

			CommandLogging.WriteLine( from, sb.ToString() );

			string name = args[0];

			FixArgs( ref args );

			Type type = FindTypeByName( name );

			if ( type == null )
			{
				from.SendMessage( "No type with that name was found." );
				return;
			}

			DateTime time = DateTime.Now;

			int built = BuildObjects( from, type, start, end, args );

			if ( built > 0 )
				from.SendMessage( "{0} object{1} generated in {2:F1} seconds.", built, built != 1 ? "s" : "", (DateTime.Now - time).TotalSeconds );
			else
				SendUsage( type, from );
		}

		public static void FixArgs( ref string[] args )
		{
			string[] old = args;
			args = new string[args.Length - 1];

			Array.Copy( old, 1, args, 0, args.Length );
		}

		public static Type FindTypeByName( string name )
		{
			Type type = null;

			Type[] types = ScriptCompiler.Assembly.GetTypes();

			for ( int i = 0; i < types.Length && type == null; ++i )
				if ( CaseInsensitiveComparer.Default.Compare( types[i].Name, name ) == 0 )
					type = types[i];

			if ( type == null )
			{
				types = Assembly.GetEntryAssembly().GetTypes();

				for ( int i = 0; i < types.Length && type == null; ++i )
					if ( CaseInsensitiveComparer.Default.Compare( types[i].Name, name ) == 0 )
						type = types[i];
			}

			return type;
		}

		public static int BuildObjects( Mobile from, Type type, Point3D start, Point3D end, string[] args )
		{
			Utility.FixPoints( ref start, ref end );

			ConstructorInfo[] ctors = type.GetConstructors();

			for ( int i = 0; i < ctors.Length; ++i )
			{
				ConstructorInfo ctor = ctors[i];

				if ( !IsConstructable( ctor ) )
					continue;

				ParameterInfo[] paramList = ctor.GetParameters();

				if ( args.Length == paramList.Length )
				{
					object[] paramValues = ParseValues( paramList, args );

					if ( paramValues == null )
						continue;

					int built = Build( from, start, end, ctor, paramValues );

					if ( built > 0 )
						return built;
				}
			}

			return 0;
		}

		public static object[] ParseValues( ParameterInfo[] paramList, string[] args )
		{
			object[] values = new object[args.Length];

			for ( int i = 0; i < args.Length; ++i )
			{
				object value = ParseValue( paramList[i].ParameterType, args[i] );

				if ( value != null )
					values[i] = value;
				else
					return null;
			}

			return values;
		}

		public static object ParseValue( Type type, string value )
		{
			try
			{
				if ( IsEnum( type ) )
				{
					return Enum.Parse( type, value, true );
				}
				else if ( IsType( type ) )
				{
					return FindTypeByName( value );
				}
				else if ( IsParsable( type ) )
				{
					return ParseParsable( type, value );
				}
				else
				{
					object obj = value;

					if ( value != null && value.StartsWith( "0x" ) )
					{
						if ( IsSignedNumeric( type ) )
							obj = Convert.ToInt64( value.Substring( 2 ), 16 );
						else if ( IsUnsignedNumeric( type ) )
							obj = Convert.ToUInt64( value.Substring( 2 ), 16 );

						obj = Convert.ToInt32( value.Substring( 2 ), 16 );
					}

					if ( obj == null && !type.IsValueType )
						return null;
					else
						return Convert.ChangeType( obj, type );
				}
			}
			catch
			{
				return null;
			}
		}

		public static int Build( Mobile from, Point3D start, Point3D end, ConstructorInfo ctor, object[] values )
		{
			try
			{
				Map map = from.Map;

				int objectCount = ((end.X - start.X) + 1) * ((end.Y - start.Y) + 1);

				if ( objectCount >= 20 )
					from.SendMessage( "Constructing {0} objects, please wait.", objectCount );

				for ( int x = start.X; x <= end.X; ++x )
				{
					for ( int y = start.Y; y <= end.Y; ++y )
					{
						object built = ctor.Invoke( values );

						if ( built is Item )
						{
							Item item = (Item)built;

							item.MoveToWorld( new Point3D( x, y, start.Z ), map );
						}
						else if ( built is Mobile )
						{
							Mobile m = (Mobile)built;

							m.Location = new Point3D( x, y, start.Z );
							m.Map = map;
						}
					}
				}

				return objectCount;
			}
			catch
			{
				return 0;
			}
		}

		public static void SendUsage( Type type, Mobile from )
		{
			ConstructorInfo[] ctors = type.GetConstructors();
			bool foundCtor = false;

			for ( int i = 0; i < ctors.Length; ++i )
			{
				ConstructorInfo ctor = ctors[i];

				if ( !IsConstructable( ctor ) )
					continue;

				if ( !foundCtor )
				{
					foundCtor = true;
					from.SendMessage( "Usage:" );
				}

				SendCtor( type, ctor, from );
			}

			if ( !foundCtor )
				from.SendMessage( "That type is not marked constructable." );
		}

		public static void SendCtor( Type type, ConstructorInfo ctor, Mobile from )
		{
			ParameterInfo[] paramList = ctor.GetParameters();

			StringBuilder sb = new StringBuilder();

			sb.Append( type.Name );

			for ( int i = 0; i < paramList.Length; ++i )
			{
				if ( i != 0 )
					sb.Append( ',' );

				sb.Append( ' ' );

				sb.Append( paramList[i].ParameterType.Name );
				sb.Append( ' ' );
				sb.Append( paramList[i].Name );
			}

			from.SendMessage( sb.ToString() );
		}

		private class AddTarget : Target
		{
			private string[] m_Args;

			public AddTarget( string[] args ) : base( -1, true, TargetFlags.None )
			{
				m_Args = args;
			}

			protected override void OnTarget( Mobile from, object o )
			{
				IPoint3D p = o as IPoint3D;

				if ( p != null )
				{
					if ( p is Item )
						p = ((Item)p).GetWorldTop();
					else if ( p is Mobile )
						p = ((Mobile)p).Location;

					Add.Invoke( from, new Point3D( p ), new Point3D( p ), m_Args );
				}
			}
		}

		[Usage( "Add <name> [params]" )]
		[Description( "Adds an item or npc by name to a targeted location. Optional constructor parameters." )]
		public static void Add_OnCommand( CommandEventArgs e )
		{
			if ( e.Length >= 1 )
				e.Mobile.Target = new AddTarget( e.Arguments );
			else
				e.Mobile.SendMessage( "Format: Add <type> [params]" );
		}

		private class TileTarget : Target
		{
			private string[] m_Args;
			private bool m_FixZ;
			private int m_FixedZ;

			public TileTarget( string[] args ) : base( -1, true, TargetFlags.None )
			{
				m_Args = args;
			}

			public TileTarget( string[] args, int fixedZ ) : base( -1, true, TargetFlags.None )
			{
				m_Args = args;
				m_FixZ = true;
				m_FixedZ = fixedZ;
			}

			protected override void OnTarget( Mobile from, object o )
			{
				IPoint3D p = o as IPoint3D;

				if ( p != null )
				{
					if ( p is Item )
						p = ((Item)p).GetWorldTop();

					from.Target = new InternalTarget( m_Args, new Point3D( p ), m_FixZ, m_FixedZ );
				}
			}

			private class InternalTarget : Target
			{
				private string[] m_Args;
				private bool m_FixZ;
				private int m_FixedZ;
				private Point3D m_Point;

				public InternalTarget( string[] args, Point3D p, bool fixZ, int fixedZ ) : base( -1, true, TargetFlags.None )
				{
					m_Args = args;
					m_Point = p;
					m_FixZ = fixZ;
					m_FixedZ = fixedZ;
				}

				protected override void OnTarget( Mobile from, object targeted )
				{
					IPoint3D p = targeted as IPoint3D;

					if ( p != null )
					{
						if ( p is Item )
							p = ((Item)p).GetWorldTop();

						Point3D rp = new Point3D( p );

						if ( m_FixZ )
						{
							m_Point.Z = m_FixedZ;
							rp.Z = m_FixedZ;
						}

						Add.Invoke( from, m_Point, rp, m_Args );
					}
				}
			}
		}

		[Usage( "Tile <name> [params]" )]
		[Description( "Tiles an item or npc by name into a targeted bounding box. Optional constructor parameters." )]
		public static void Tile_OnCommand( CommandEventArgs e )
		{
			if ( e.Length >= 1 )
				e.Mobile.Target = new TileTarget( e.Arguments );
			else
				e.Mobile.SendMessage( "Format: Add <type> [params]" );
		}

		[Usage( "TileRXYZ <x> <y> <w> <h> <z> <name> [params]" )]
		[Description( "Tiles an item or npc by name into a given bounding box, (x, y) parameters are relative to your characters position. Optional constructor parameters." )]
		public static void TileRXYZ_OnCommand( CommandEventArgs e )
		{
			if ( e.Length >= 6 )
			{
				Point3D p = new Point3D( e.Mobile.X + e.GetInt32( 0 ), e.Mobile.Y + e.GetInt32( 1 ), e.Mobile.Z + e.GetInt32( 4 ) );
				Point3D p2 = new Point3D( p.X + e.GetInt32( 2 ) - 1, p.Y + e.GetInt32( 3 ) - 1, e.GetInt32( 4 ) );

				string[] subArgs = new string[e.Length - 5];

				for ( int i = 0; i < subArgs.Length; ++i )
					subArgs[i] = e.Arguments[i + 5];

				Add.Invoke( e.Mobile, p, p2, subArgs );
			}
			else
			{
				e.Mobile.SendMessage( "Format: TileRXYZ <x> <y> <w> <h> <z> <type> [params]" );
			}
		}

		[Usage( "TileXYZ <x> <y> <w> <h> <z> <name> [params]" )]
		[Description( "Tiles an item or npc by name into a given bounding box. Optional constructor parameters." )]
		public static void TileXYZ_OnCommand( CommandEventArgs e )
		{
			if ( e.Length >= 6 )
			{
				Point3D p = new Point3D( e.GetInt32( 0 ), e.GetInt32( 1 ), e.GetInt32( 4 ) );
				Point3D p2 = new Point3D( p.X + e.GetInt32( 2 ) - 1, p.Y + e.GetInt32( 3 ) - 1, e.GetInt32( 4 ) );

				string[] subArgs = new string[e.Length - 5];

				for ( int i = 0; i < subArgs.Length; ++i )
					subArgs[i] = e.Arguments[i + 5];

				Add.Invoke( e.Mobile, p, p2, subArgs );
			}
			else
			{
				e.Mobile.SendMessage( "Format: TileXYZ <x> <y> <w> <h> <z> <type> [params]" );
			}
		}

		[Usage( "TileZ <z> <name> [params]" )]
		[Description( "Tiles an item or npc by name into a targeted bounding box at a fixed Z location. Optional constructor parameters." )]
		public static void TileZ_OnCommand( CommandEventArgs e )
		{
			if ( e.Length >= 2 )
			{
				string[] subArgs = new string[e.Length - 1];

				for ( int i = 0; i < subArgs.Length; ++i )
					subArgs[i] = e.Arguments[i + 1];

				e.Mobile.Target = new TileTarget( subArgs, e.GetInt32( 0 ) );
			}
			else
			{
				e.Mobile.SendMessage( "Format: TileZ <z> <type> [params]" );
			}
		}

		private static Type m_ConstructableType = typeof( ConstructableAttribute );

		public static bool IsConstructable( ConstructorInfo ctor )
		{
			return ctor.IsDefined( m_ConstructableType, false );
		}

		private static Type m_EnumType = typeof( Enum );

		public static bool IsEnum( Type type )
		{
			return type.IsSubclassOf( m_EnumType );
		}

		private static Type m_TypeType = typeof( Type );

		public static bool IsType( Type type )
		{
			return ( type == m_TypeType || type.IsSubclassOf( m_TypeType ) );
		}

		private static Type m_ParsableType = typeof( ParsableAttribute );

		public static bool IsParsable( Type type )
		{
			return type.IsDefined( m_ParsableType, false );
		}

		private static Type[] m_ParseTypes = new Type[]{ typeof( string ) };
		private static object[] m_ParseArgs = new object[1];

		public static object ParseParsable( Type type, string value )
		{
			MethodInfo method = type.GetMethod( "Parse", m_ParseTypes );

			m_ParseArgs[0] = value;

			return method.Invoke( null, m_ParseArgs );
		}

		private static Type[] m_SignedNumerics = new Type[]
			{
				typeof( Int64 ),
				typeof( Int32 ),
				typeof( Int16 ),
				typeof( SByte )
			};

		public static bool IsSignedNumeric( Type type )
		{
			for ( int i = 0; i < m_SignedNumerics.Length; ++i )
				if ( type == m_SignedNumerics[i] )
					return true;

			return false;
		}

		private static Type[] m_UnsignedNumerics = new Type[]
			{
				typeof( UInt64 ),
				typeof( UInt32 ),
				typeof( UInt16 ),
				typeof( Byte )
			};

		public static bool IsUnsignedNumeric( Type type )
		{
			for ( int i = 0; i < m_UnsignedNumerics.Length; ++i )
				if ( type == m_UnsignedNumerics[i] )
					return true;

			return false;
		}
	}
}