using System;
using Server;
using Server.Gumps;
using Server.Network;
using Server.Menus;
using Server.Menus.Questions;

namespace Server.Engines.Help
{
	public class ContainedMenu : QuestionMenu
	{
		private Mobile m_From;

		public ContainedMenu( Mobile from ) : base( "You already have an open help request. We will have someone assist you as soon as possible.  What would you like to do?", new string[]{ "Leave my old help request like it is.", "Remove my help request from the queue." } )
		{
			m_From = from;
		}

		public override void OnCancel( NetState state )
		{
			m_From.SendLocalizedMessage( 1005306, "", 0x35 ); // Help request unchanged.
		}

		public override void OnResponse( NetState state, int index )
		{
			if ( index == 0 )
			{
				m_From.SendLocalizedMessage( 1005306, "", 0x35 ); // Help request unchanged.
			}
			else if ( index == 1 )
			{
				PageEntry entry = PageQueue.GetEntry( m_From );

				if ( entry != null && entry.Handler == null )
				{
					m_From.SendLocalizedMessage( 1005307, "", 0x35 ); // Removed help request.
					PageQueue.Remove( entry );
				}
				else
				{
					m_From.SendLocalizedMessage( 1005306, "", 0x35 ); // Help request unchanged.
				}
			}
		}
	}

	public class HelpGump : Gump
	{
		public static void Initialize()
		{
			EventSink.HelpRequest += new HelpRequestEventHandler( EventSink_HelpRequest );
		}

		private static void EventSink_HelpRequest( HelpRequestEventArgs e )
		{
			foreach ( Gump g in e.Mobile.NetState.Gumps )
			{
				if ( g is HelpGump )
					return;
			}

			if ( PageQueue.Contains( e.Mobile ) )
				e.Mobile.SendMenu( new ContainedMenu( e.Mobile ) );
			else
				e.Mobile.SendGump( new HelpGump( e.Mobile ) );
		}

		private static bool IsYoung( Mobile m )
		{
			Server.Accounting.Account acct = m.Account;

			return ( acct != null && (DateTime.Now - acct.Created) < TimeSpan.FromHours( 24.0 ) );
		}

		public static bool CheckCombat( Mobile m )
		{
			for ( int i = 0; i < m.Aggressed.Count; ++i )
			{
				AggressorInfo info = (AggressorInfo)m.Aggressed[i];

				if ( DateTime.Now - info.LastCombatTime < TimeSpan.FromSeconds( 30.0 ) )
					return true;
			}

			return false;
		}

		public HelpGump( Mobile from ) : base( 0, 0 )
		{
			from.CloseGump( typeof( HelpGump ) );

			bool isYoung = IsYoung( from );

			Add( new GumpBackground( 100, 25, 440, 430, 2600 ) );

			Add( new GumpPage( 0 ) );

			Add( new GumpHtmlLocalized( 150, 50, 360, 40, 1001002, false, false ) ); // Ultima Online Help Menu
			Add( new GumpButton( 425, 415, 2073, 2072, 0, GumpButtonType.Reply, 0 ) ); // Close

			if ( isYoung )
			{
				Add( new GumpPage( 1 ) );

				Add( new GumpButton( 130, 75, 5540, 5541, 8, GumpButtonType.Reply, 0 ) ); 
				Add( new GumpHtml( 160, 75, 350, 58, "<BODY><BASEFONT COLOR=BLACK><u>Young Player Haven Transport.</u> Select this option if you want to be transported to <BASEFONT COLOR=#00FF00>Haven <BASEFONT COLOR=BLACK>.</BODY>", true, true ) );

				Add( new GumpButton( 130, 140, 5540, 5541, 1, GumpButtonType.Reply, 2 ) );
				Add( new GumpHtml( 160, 140, 350, 58, "<u>General question about Ultima Online.</u> Select this option if you have a general gameplay question, need help learning to use a skill, or if you would like to search the UO Knowledge Base.", true, true ) );

				Add( new GumpButton( 130, 205, 5540, 5541, 2, GumpButtonType.Reply, 0 ) );
				Add( new GumpHtml( 160, 205, 350, 58, "<u>My character is physically stuck in the game.</u> This choice only covers cases where your character is physically stuck in a location they cannot move out of. This option will only work two times in 24 hours.", true, true ) );

				Add( new GumpButton( 130, 270, 5540, 5541, 0, GumpButtonType.Page, 3 ) );
				Add( new GumpHtml( 160, 270, 350, 58, "<u>Another player is harassing me.</u> Another player is verbally harassing your character. When you select this option you will be sending a text log to Origin Systems. To see what constitutes harassment please visit http://support.uo.com/gm_9.html.", true, true ) );

				Add( new GumpButton( 130, 335, 5540, 5541, 0, GumpButtonType.Page, 2 ) );
				Add( new GumpHtml( 160, 335, 350, 58, "<u>Other.</u> If you are experiencing a problem in the game that does not fall into one of the other categories or is not addressed on the Support web page (located at http://support.uo.com), please use this option.", true, true ) );
			}
			else
			{
				Add( new GumpPage( 1 ) );

				Add( new GumpButton( 130, 90, 5540, 5541, 1, GumpButtonType.Reply, 2 ) );
				Add( new GumpHtml( 160, 90, 350, 74, "<u>General question about Ultima Online.</u> Select this option if you have a general gameplay question, need help learning to use a skill, or if you would like to search the UO Knowledge Base.", true, true ) );

				Add( new GumpButton( 130, 170, 5540, 5541, 2, GumpButtonType.Reply, 0 ) );
				Add( new GumpHtml( 160, 170, 350, 74, "<u>My character is physically stuck in the game.</u> This choice only covers cases where your character is physically stuck in a location they cannot move out of. This option will only work two times in 24 hours.", true, true ) );

				Add( new GumpButton( 130, 250, 5540, 5541, 0, GumpButtonType.Page, 3 ) );
				Add( new GumpHtml( 160, 250, 350, 74, "<u>Another player is harassing me.</u> Another player is verbally harassing your character. When you select this option you will be sending a text log to Origin Systems. To see what constitutes harassment please visit http://support.uo.com/gm_9.html.", true, true ) );

				Add( new GumpButton( 130, 330, 5540, 5541, 0, GumpButtonType.Page, 2 ) );
				Add( new GumpHtml( 160, 330, 350, 74, "<u>Other.</u> If you are experiencing a problem in the game that does not fall into one of the other categories or is not addressed on the Support web page (located at http://support.uo.com), please use this option.", true, true ) );
			}

			Add( new GumpPage( 2 ) );

			Add( new GumpButton( 130, 90, 5540, 5541, 3, GumpButtonType.Reply, 0 ) );
			Add( new GumpHtml( 160, 90, 350, 74, "<u>Report a bug or contact Origin.</u> Use this option to launch your web browser and mail in a bug report. Your report will be read by our Quality Assurance Staff. We apologize for not being able to reply to individual reports. ", true, true ) );

			Add( new GumpButton( 130, 170, 5540, 5541, 4, GumpButtonType.Reply, 0 ) );
			Add( new GumpHtml( 160, 170, 350, 74, "<u>Suggestion for the Game.</u> If you'd like to make a suggestion for the game, it should be directed to the Development Team Members who participate in the discussion forums on the UO.Com web site. Choosing this option will take you to the Discussion Forums. ", true, true ) );

			Add( new GumpButton( 130, 250, 5540, 5541, 5, GumpButtonType.Reply, 0 ) );
			Add( new GumpHtml( 160, 250, 350, 74, "<u>Account Management</u> For questions regarding your account such as forgotten passwords, payment options, account activation, and account transfer, please choose this option.", true, true ) );

			Add( new GumpButton( 130, 330, 5540, 5541, 6, GumpButtonType.Reply, 0 ) );
			Add( new GumpHtml( 160, 330, 350, 74, "<u>Other.</u> If you are experiencing a problem in the game that does not fall into one of the other categories or is not addressed on the Support web page (located at http://support.uo.com), and requires in-game assistance, use this option. ", true, true ) );

			Add( new GumpPage( 3 ) );

			Add( new GumpHtmlLocalized( 130, 90, 380, 250, 1001013, true, true ) ); /* You have asked to file a complaint on another player for harassment in Ultima Online.
																					 * A logfile will be reviewed by our support staff if you send in this page, and you will
																					 * receive a message through mail telling you of the status of your complaint.
																					 * Please make sure you have your proper e-mail address on record with us.
																					 * Harassment is defined on our website at :
																					 * <A HREF="http://support.uo.com/gm_9.html">http://support.uo.com/gm_9.html</A><BR>
																					 * <U><CENTER>Please review the following to make sure you wish to continue with this page:</CENTER></U><BR>
																					 * 1. You have asked the player to stop and they have continued. <BR>
																					 * 2. You have tried to remove yourself from the situation. <BR>
																					 * 3. You have done nothing to instigate or further encourage the harassment. <BR>
																					 * 4. You have read and understand Origin's definition of harassment. <BR>
																					 * Do you wish to continue with this call?
																					 */

			Add( new GumpButton( 150, 350, 5540, 5541, 7, GumpButtonType.Reply, 0 ) );
			Add( new GumpHtmlLocalized( 180, 350, 335, 40, 1001014, false, false ) ); // YES - I want to send a harassment log to the GMs.

			Add( new GumpButton( 150, 385, 5540, 5541, 0, GumpButtonType.Page, 1 ) );
			Add( new GumpHtmlLocalized( 180, 385, 335, 40, 1001015, false, false ) ); // NO  - I meant to ask for help with another matter.
		}

		public override void OnResponse( NetState state, RelayInfo info )
		{
			Mobile from = state.Mobile;

			switch ( info.ButtonID )
			{
				case 0: // Close/Cancel
				{
					from.SendLocalizedMessage( 501235, "", 0x35 ); // Help request aborted.

					break;
				}
				case 1: // General question
				{
					from.SendLocalizedMessage( 501232, "", 0x35 ); // Please enter a brief description (up to 40 characters) of your problem:
					from.Prompt = new PagePrompt( PageType.Question );

					break;
				}
				case 2: // Stuck
				{
					if ( from.Region is Server.Regions.Jail )
					{
						from.SendLocalizedMessage( 1041530, "", 0x35 ); // You'll need a better jailbreak plan then that!
					}
					else if ( from.CanUseStuckMenu() && from.Region.CanUseStuckMenu( from ) && !CheckCombat( from ) && !from.Frozen && !from.Criminal && from.Kills < 5 )
					{
						StuckMenu menu = new StuckMenu( from, from, true );

						menu.BeginClose();

						from.SendGump( menu );
					}
					else
					{
						from.SendLocalizedMessage( 501232, "", 0x35 ); // Please enter a brief description (up to 40 characters) of your problem:
						from.Prompt = new PagePrompt( PageType.Stuck );
					}

					break;
				}
				case 3: // Report bug or contact Origin
				{
					from.SendLocalizedMessage( 501232, "", 0x35 ); // Please enter a brief description (up to 40 characters) of your problem:
					from.Prompt = new PagePrompt( PageType.Bug );

					break;
				}
				case 4: // Game suggestion
				{
					from.SendLocalizedMessage( 501232, "", 0x35 ); // Please enter a brief description (up to 40 characters) of your problem:
					from.Prompt = new PagePrompt( PageType.Suggestion );

					break;
				}
				case 5: // Account management
				{
					from.SendLocalizedMessage( 501232, "", 0x35 ); // Please enter a brief description (up to 40 characters) of your problem:
					state.Mobile.Prompt = new PagePrompt( PageType.Account );

					break;
				}
				case 6: // Other
				{
					from.SendLocalizedMessage( 501232, "", 0x35 ); // Please enter a brief description (up to 40 characters) of your problem:
					from.Prompt = new PagePrompt( PageType.Other );

					break;
				}
				case 7: // Harassment
				{
					from.SendLocalizedMessage( 501232, "", 0x35 ); // Please enter a brief description (up to 40 characters) of your problem:
					from.Prompt = new PagePrompt( PageType.Harassment );

					break;
				}
				case 8: // Young player transport
				{
					if ( IsYoung( from ) )
					{
						if ( from.Region.Name == "Haven" )
						{
							from.SendLocalizedMessage( 1041529 ); // You're already in Haven
						}
						else
						{
							from.Map = Map.Trammel;
							from.Location = new Point3D( 3618, 2587, 0 );
						}
					}

					break;
				}
			}
		}
	}
}